import { readFileSync } from 'fs'

const DECRYPTION_KEY = 811589153

function mixing(numbers: number[], times = 1): number[] {
  const indexes = numbers.map((_, i) => i)
  const mixed = [...numbers]
  const size = numbers.length

  for (let time = 0; time < times; time++) {
    for (let i = 0; i < size; i++) {
      const value = numbers[i]
      const mixedIndex = indexes.indexOf(i)
      const valueCircular = value % (size - 1)
      const indexTo = mixedIndex + valueCircular
      let moveIndexTo = indexTo

      if (indexTo < 0) {
        moveIndexTo = size - 1 + indexTo
      } else if (indexTo > size - 1) {
        moveIndexTo = indexTo - size + 1
      }

      if (mixedIndex < moveIndexTo) {
        mixed.splice(moveIndexTo + 1, 0, value)
        indexes.splice(moveIndexTo + 1, 0, i)

        mixed.splice(mixedIndex, 1)
        indexes.splice(mixedIndex, 1)
      } else if (mixedIndex > moveIndexTo) {
        mixed.splice(moveIndexTo, 0, value)
        indexes.splice(moveIndexTo, 0, i)

        mixed.splice(mixedIndex + 1, 1)
        indexes.splice(mixedIndex + 1, 1)
      }
    }
  }

  return mixed
}

function coordinates(mixed: number[]): number {
  const size = mixed.length

  const zeroIndex = mixed.indexOf(0)
  const x = mixed[(zeroIndex + 1000) % size]
  const y = mixed[(zeroIndex + 2000) % size]
  const z = mixed[(zeroIndex + 3000) % size]

  return x + y + z
}

function solvePart1(lines: string[]): number {
  const numbers = lines.map((line) => parseInt(line, 10))
  const mixed = mixing(numbers)
  return coordinates(mixed)
}

function solvePart2(lines: string[]): number {
  const numbers = lines.map((line) => parseInt(line, 10))
  const decrypted = numbers.map((n) => n * DECRYPTION_KEY)
  const mixed = mixing(decrypted, 10)

  return coordinates(mixed)
}

function run() {
  const input = process.argv.slice(2)[0]
  const file = readFileSync(input, 'utf8')
  const lines = file.split('\n').slice(0, -1)

  const solution1 = solvePart1(lines)
  console.log(`part1: ${solution1}`)

  const solution2 = solvePart2(lines)
  console.log(`part2: ${solution2}`)
}

run()
