import { FileHandle, open } from 'fs/promises'

function characterPriority(item: string): number {
  const alpha = 'abcdefghijklmnopqrstuvwxyz'
  const priorityList = `${alpha}${alpha.toUpperCase()}`
  const priority = priorityList.indexOf(item) + 1

  if (!priority) {
    throw new Error('invalid item')
  }

  return priority
}

function findCommonCharacter(items: string[]): string {
  const [first, rest] = [items[0], items.slice(1)]
  const found = first.split('').find((char) => rest.every((x) => x.includes(char)))

  if (!found) {
    throw new Error('No common character found')
  }

  return found
}

function scorePriority(items: string[]): number {
  const commonChar = findCommonCharacter(items)
  return characterPriority(commonChar)
}

async function scorePriorities(file: FileHandle): Promise<number> {
  const size = 3
  const lines = []
  let totalScore = 0

  for await (const line of file.readLines()) {
    lines.push(line)
    if (lines.length === size) {
      totalScore += scorePriority(lines)
      lines.splice(0, size)
    }
  }

  return totalScore
}

async function run() {
  const input = process.argv.slice(2)[0]
  const fd = await open(input)
  const totalScore = await scorePriorities(fd)

  console.log(`solution: ${totalScore}`)
}

run()
