import { FileHandle, open } from 'fs/promises'

function isOverlapping(one: number[], two: number[]): boolean {
  return (
    // fully contained
    (one[0] <= two[0] && one[1] >= two[1]) ||
    // early overlap
    (one[0] <= two[0] && one[1] >= two[0]) ||
    // late overlap
    (one[0] <= two[1] && one[1] >= two[1])
  )
}

function areAssignmentsOverlapping(assignments: string): boolean {
  const [one, two] = assignments.split(',').map((x) => x.split('-').map((x) => parseInt(x)))

  return isOverlapping(one, two) || isOverlapping(two, one)
}

async function scorePriorities(file: FileHandle): Promise<number> {
  let containedPartners = 0

  for await (const line of file.readLines()) {
    containedPartners += areAssignmentsOverlapping(line) ? 1 : 0
  }

  return containedPartners
}

async function run() {
  const input = process.argv.slice(2)[0]
  const fd = await open(input)
  const totalScore = await scorePriorities(fd)

  console.log(`solution: ${totalScore}`)
}

run()
