import { FileHandle, open } from 'fs/promises'

type Instruction = {
  move: number
  from: number
  to: number
}

function isCrates(line: string): boolean {
  return line.trim().startsWith('[')
}

function isMove(line: string): boolean {
  return line.trim().startsWith('move')
}

function appendCrateRow(crates: string[][], crateRow: string[]) {
  crateRow.forEach((c, i) => {
    if (crates[i] === undefined) {
      crates[i] = []
    }
    if (c.trim()) {
      crates[i].push(c)
    }
  })
}

function parseCrateLine(line: string): string[] {
  return line
    .split('')
    .reduce((p, c, i) => (i % 4 === 1 ? `${p}${c}` : `${p}`), '')
    .split('')
}

function parseMoveLine(line: string): Instruction {
  const parsed = line.match(/^\s*move\s*(\d+)\s*from\s*(\d+)\s*to\s*(\d+)\s*$/)
  if (parsed) {
    const [, move, from, to] = parsed
    return { move: +move, from: +from, to: +to }
  }
  throw new Error(`invalid move line: ${line}`)
}

function moveCrates(crates: string[][], instruction: Instruction) {
  const moved = []
  for (let i = 0; i < instruction.move; i++) {
    const shift = crates[instruction.from - 1].shift()
    moved.push(shift || '')
  }
  moved.reverse().forEach((crate) => {
    crates[instruction.to - 1].unshift(crate)
  })
}

async function solve(file: FileHandle): Promise<void> {
  const crates: string[][] = []

  for await (const line of file.readLines()) {
    if (isCrates(line)) {
      const crateRow = parseCrateLine(line)
      appendCrateRow(crates, crateRow)
    } else if (isMove(line)) {
      const instruction = parseMoveLine(line)
      moveCrates(crates, instruction)
    }
  }

  console.log(`solution: ${crates.map((x) => x.join(''))}`)
}

async function run() {
  const input = process.argv.slice(2)[0]
  const fd = await open(input)
  await solve(fd)
}

run()
