import { FileHandle, open } from 'fs/promises'

function sceneToTheRight(row: number, column: number, rows: number[][]): number {
  const numColumns = rows[row].length
  if (column === numColumns - 1) {
    return 0
  } else {
    const height = rows[row][column]
    const blockedAt = rows[row].slice(column + 1).findIndex((cell) => cell >= height)
    return blockedAt === -1 ? numColumns - 1 - column : blockedAt + 1
  }
}

function sceneToTheLeft(row: number, column: number, rows: number[][]): number {
  if (column === 0) {
    return 0
  } else {
    const height = rows[row][column]
    const blockedAt = rows[row]
      .slice(0, column)
      .reverse()
      .findIndex((cell) => cell >= height)
    return blockedAt === -1 ? column : blockedAt + 1
  }
}

function sceneToTheNorth(row: number, column: number, rows: number[][]): number {
  if (row === 0) {
    return 0
  } else {
    const height = rows[row][column]
    const blockedAt = rows
      .slice(0, row)
      .reverse()
      .findIndex((row) => row[column] >= height)
    return blockedAt === -1 ? row : blockedAt + 1
  }
}

function sceneToTheSouth(row: number, column: number, rows: number[][]): number {
  const numRows = rows.length
  if (row === numRows - 1) {
    return 0
  } else {
    const height = rows[row][column]
    const blockedAt = rows.slice(row + 1).findIndex((row) => row[column] >= height)
    return blockedAt === -1 ? numRows - 1 - row : blockedAt + 1
  }
}

function calculateSceneOfTrees(rows: number[][]): number[][] {
  return rows.map((row, rowIndex) => {
    return row.map((_, columnIndex) => {
      const right = sceneToTheRight(rowIndex, columnIndex, rows)
      const left = sceneToTheLeft(rowIndex, columnIndex, rows)
      const north = sceneToTheNorth(rowIndex, columnIndex, rows)
      const south = sceneToTheSouth(rowIndex, columnIndex, rows)
      return left * right * north * south
    })
  })
}

function maxScene(scene: number[][]): number {
  return scene.reduce(
    (rowMax, rows) =>
      Math.max(
        rowMax,
        rows.reduce((columnMax, calc) => Math.max(columnMax, calc), 0)
      ),
    0
  )
}

async function solve(file: FileHandle): Promise<void> {
  const rows: number[][] = []

  for await (const line of file.readLines()) {
    rows.push(line.split('').map((c) => +c))
  }

  const scene = calculateSceneOfTrees(rows)

  console.log(`solution: ${maxScene(scene)}`)
}

async function run() {
  const input = process.argv.slice(2)[0]
  const fd = await open(input)
  await solve(fd)
}

run()
